#include "Arduino.h"
#include "WiFi.h"
#include "WiFiUdp.h"
#include "OSCMessage.h"
#include "OSCBundle.h"
#include "SPI.h"
#include "SD.h"
#include "Adafruit_VS1053.h"
#include "HX711.h"

#define BL_ON 0

#if BL_ON
#include "BLEDevice.h"
#include "BLEScan.h"

#define blName "Sessel"
#endif

  
// WiFi stuff
#define ssid ""
#define pwd ""

// OSC
#define host "192.168.21.232"
#define port  3030
#define localPort 4040

// feather music maker
#define maxFileNameLength 20
#define VS1053_RESET   -1     // VS1053 reset pin (not used!)
#define VS1053_CS      32     // VS1053 chip select pin (output)
#define VS1053_DCS     33     // VS1053 Data/command select pin (output)
#define CARDCS         14     // Card chip select pin
#define VS1053_DREQ    15     // VS1053 Data request, ideally an Interrupt pin
/*
Miso --------> IO19
Mosi --------> IO23
Sck --------> IO18

Sd Cs -------> IO14
MP3 Cs ----> IO32
DReq -------> IO15
XdCs -------> IO33
*/


// load sensors 
#define DOUT  21
#define CLK  22

WiFiUDP Udp;
OSCErrorCode error;
HX711 scale;
float calibration_factor = -10380;
const float threshold = 0.1;

Adafruit_VS1053_FilePlayer musicPlayer = 
  Adafruit_VS1053_FilePlayer(VS1053_RESET, VS1053_CS, VS1053_DCS, VS1053_DREQ, CARDCS);

void bgTask(void * parameter ) {
    scale.begin(DOUT, CLK);
    scale.set_scale(calibration_factor);
    scale.tare(); //Reset the scale to 0
    float w = scale.get_units(4); 

     while(true){
	float wNew = scale.get_units(4);
	if (abs(w-wNew) > threshold){
		OSCMessage msgOut("/sessel/gewicht");
  		msgOut.add(w);
  		Udp.beginPacket(host, port);
  		msgOut.send(Udp);
  		Udp.endPacket();
		w = wNew;
	}
      	vTaskDelay(100);}
}

void setup() {
    Serial.begin(115200);
    delay(500);

    if (! musicPlayer.begin()) { // initialise the music player
     	Serial.println(F("Couldn't find VS1053"));
    }

    if (!SD.begin(CARDCS)) {
    	Serial.println(F("SD failed, or not present"));
     }

    musicPlayer.setVolume(10,10);
    
   

    #if BL_ON
    BLEDevice::init(blName);
    #endif

    WiFi.begin(ssid, pwd);
    while (WiFi.status() != WL_CONNECTED) {
        Serial.print(".");
        delay(500);
    }
    Serial.print("WiFi connected, IP = ");
    Serial.println(WiFi.localIP());
    delay(1500);


    Udp.begin(localPort);

    xTaskCreatePinnedToCore(
    bgTask, 
    "background task",
    4000,
    NULL,
    1,
    NULL,
    0 );

}

void play(OSCMessage & msg){
	char fileName[maxFileNameLength];
	char * file = fileName;
	msg.getString(0, file, maxFileNameLength);
	Serial.println(file);
	musicPlayer.startPlayingFile(file);
}

void stop(OSCMessage & msg){
	Serial.println("stop");
    	musicPlayer.stopPlaying();
}

void pause(OSCMessage & msg){
	bool state = msg.getInt(0);
	Serial.print("pause ");
	Serial.println(state);
      	musicPlayer.pausePlaying(state);
}

void togglePause(OSCMessage & msg){
	Serial.println("togglePause");
     if (! musicPlayer.paused()) {
        musicPlayer.pausePlaying(true);
      } else { 
        musicPlayer.pausePlaying(false);
      }
}

void setVolume(OSCMessage & msg){
	int vol = msg.getInt(0);
	Serial.print("set Volume to ");
	Serial.println(vol);
	musicPlayer.setVolume(vol,vol);
}


void loop() {
  
  musicPlayer.feedBuffer();
  OSCMessage msg;
  int size = Udp.parsePacket();

  if (size > 0) {
    while (size--) {
      msg.fill(Udp.read());
    }
    if (!msg.hasError()) {
      msg.dispatch("/play", play);
      msg.dispatch("/stop", stop);
      msg.dispatch("/pause", pause);
      msg.dispatch("/togglePause", togglePause);
      msg.dispatch("/setVolume", setVolume);
    } else {
      error = msg.getError();
      Serial.print("error: ");
      Serial.println(error);
    }
  }
  delay(10);

  
}
